package com.trifork.sosigw.sts.servlet;

import com.trifork.sosigw.config.StaticConfigImpl;
import com.trifork.sosigw.util.Util;
import com.trifork.sosigw.util.timing.Timer;
import dk.sdsd.nsp.slalog.api.SLALogItem;
import org.apache.log4j.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

public class CreateIdCardFromBSTServlet extends HttpServlet {
    public static final Logger log = Logger.getLogger(CreateIdCardFromBSTServlet.class);

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (StaticConfigImpl.getInstance().isBootstrapRestrictedMode()) {
            throw new IllegalStateException("SOSIGW running in BootstrapRestrictedMode. No webservice access allowed");
        }

        String timingOperationName = "handleRequest";
        try {
            Timer.start(CreateIdCardFromBSTServlet.class, timingOperationName);
            handleRequest(req, resp);
        } finally {
            Timer.stop(CreateIdCardFromBSTServlet.class, timingOperationName);
        }
    }

    private void handleRequest(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        Util.setActiveHttpRequest(req);
        SLALogItem slaLogItem = Util.getSLALogger().createLogItem("CreateIdCardFromBSTServlet.handleRequest()", "1");
        slaLogItem.setClientIP(req);
        Util.setSLALogItem(slaLogItem);
        log.info("SLA Log Item: "+slaLogItem);

        try {
            new CreateIdCardFromBSTRequestHandler(req, resp).handleRequest();
        } catch (ServletException e) {
            slaLogItem.setCallResultError("Servlet error: "+ e.getMessage());
            throw e;
        } catch (InvalidRequestException e) {
            log.error(e.getMessage(), e);
            slaLogItem.setCallResultError("Invalid request error: " + e.getMessage());
            resp.sendError(400, e.getMessage());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            slaLogItem.setCallResultError("Internal error: " + e.getMessage());
            throw new ServletException("Internal error: " + e.getMessage(), e);
        } finally {
            try {
                slaLogItem.store();
            } catch (Exception e) {
                log.error("Caught exception while storing SLA Log item " + slaLogItem, e);
            }
            Util.setSLALogItem(null);
            Util.setActiveHttpRequest(null);
        }
    }
}
