package com.trifork.sosigw.inttest;

import com.trifork.sosigw.util.Util;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;

import java.io.IOException;
import java.io.InputStream;

import static org.junit.Assert.fail;

public abstract class AbstractIT {
    protected int statusCode;

    protected String callServer(String requestBody, String serverUrl, RequestHeader ... headers) throws IOException {
        return callServer(requestBody, serverUrl, true, headers);
    }

    protected String callServer(String requestBody, String serverUrl, boolean failOnError, RequestHeader ... headers) throws IOException {
        HttpClient httpClient = new HttpClient();
        PostMethod postMethod = new PostMethod(serverUrl);
        for (RequestHeader header : headers) {
            postMethod.addRequestHeader(header.getName(), header.getValue());
        }
        postMethod.setRequestEntity(new StringRequestEntity(requestBody, "text/xml", "utf-8"));
        httpClient.executeMethod(postMethod);
        String response = readResponse(postMethod);

        statusCode = postMethod.getStatusCode();
        if (statusCode != 200 && failOnError) {
            fail("Status code: " + statusCode + " - status text: " + postMethod.getStatusText()
                    + " - XML response:\n" + response);
        }

        return response;
    }

    private String readResponse(PostMethod postMethod) throws IOException {
        final InputStream responseStream = postMethod.getResponseBodyAsStream();

        return Util.readAllFromStreamAsString(responseStream);
    }

    protected class RequestHeader {
        private final String name, value;

        public RequestHeader(String name, String value) {
            this.name = name;
            this.value = value;
        }

        public String getName() {
            return name;
        }

        public String getValue() {
            return value;
        }
    }

}
