package com.trifork.sosigw.ws;

import com.trifork.sosigw.config.StaticConfigImpl;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.UserIDCard;
import dk.sosi.seal.model.constants.NameSpaces;
import dk.sosi.seal.model.constants.SOAPTags;
import dk.sosi.seal.modelbuilders.IDCardModelBuilder;
import dk.sosi.seal.xml.XmlUtil;
import oasis.names.tc.saml._2_0.assertion.AssertionType;
import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.InputStream;
import java.util.Properties;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class IDCardToSamlAssertionTest {

    @Test
    public void testBuild() throws JAXBException {
        StaticConfigImpl.initSettings();
        UserIDCard userIDCard = getUserIDCardFromResources();

        AssertionType assertionType = IDCardToSamlAssertion.build(userIDCard);
        assertNotNull(assertionType);
        Security security = IdCardTestUtils.makeIDCardInSecurity();
        security.setAssertion(assertionType);

        Element idCardElement = marshalIdCard(security);
        assertNotNull(new IDCardModelBuilder().buildModel(idCardElement));
    }

    private UserIDCard getUserIDCardFromResources() {
        InputStream userIdCardStream = this.getClass().getResourceAsStream("userIdCard.xml");
        Document userIdCardDoc = XmlUtil.readXml(new Properties(), new InputSource(userIdCardStream), false);
        IDCard idCard = new IDCardModelBuilder().buildModel(userIdCardDoc);
        assertTrue(idCard instanceof UserIDCard);
        return (UserIDCard) idCard;
    }

    private Element marshalIdCard(Security security) throws JAXBException {
        JAXBContext jaxbContext = JAXBContext.newInstance(Security.class);
        Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
        Document doc = XmlUtil.createEmptyDocument();
        Element idCardElement = doc.createElementNS(NameSpaces.SOAP_SCHEMA, SOAPTags.ENVELOPE_PREFIXED);
        jaxbMarshaller.marshal(security, idCardElement);
        return idCardElement;
    }
}