/*
* The MIT License
*
* Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* $HeadURL$
* $Id$
*/
package com.trifork.sosigw.ws;

import com.trifork.sosigw.api.Constants;
import com.trifork.sosigw.testclient.ServiceStubClient;
import dk.medcom.dgws._2006._04.dgws_1_0.Header;
import dk.sosi.seal.model.constants.MedcomAttributes;
import org.junit.Before;
import org.junit.Test;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;

import javax.xml.soap.DetailEntry;
import javax.xml.ws.soap.SOAPFaultException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class ProxyRequestTest extends AbstractIntegrationTests {

    @Before
    public void setup() throws Exception {
        createSignedIdCard();
    }

    @Test(timeout = 60000)
    public void testProxyRequest() throws Exception {
        int responsePayloadSize = 10;

        System.out.println("Test Client, payload size: " + responsePayloadSize);
        String requestPayload = "small payload";

        ServiceStubClient client = new ServiceStubClient("localhost:8586");

        Header medcomHeader = null;
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();

        String result = client.doInvoke(medcomHeader, securityHeader, requestPayload, responsePayloadSize, 0, SERVER_URL + "/proxy/soap-request", false);

        System.out.println("Got: " + (result == null ? null : result.length()) + " bytes");
    }

    @Test(timeout = 60000)
    public void testProxyRequestGzip() throws Exception {
        int responsePayloadSize = 10;

        System.out.println("Test Client, payload size: " + responsePayloadSize);
        String requestPayload = "small payload";

        ServiceStubClient client = new ServiceStubClient("localhost:8586");

        Header medcomHeader = null;
        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();

        String result = client.doInvoke(medcomHeader, securityHeader, requestPayload, responsePayloadSize, 0, SERVER_URL + "/proxy/soap-request", true);

        System.out.println("Got: " + (result == null ? null : result.length()) + " bytes");
    }

    @Test(timeout = 60000)
    public void testProxyRequestWrongServiceHost() throws Exception {
        int responsePayloadSize = 10;

        System.out.println("Test Client, payload size: " + responsePayloadSize);
        String requestPayload = "small payload";

        ServiceStubClient client = new ServiceStubClient("NOHOST");

        Header medcomHeader = IdCardTestUtils.makeMedcomHeader("clientFlowID", "clientMessageID", "clientFlowStatus");

        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();

        try {
            client.doInvoke(medcomHeader, securityHeader, requestPayload, responsePayloadSize, 0, SERVER_URL + "/proxy/soap-request", false);
        } catch (SOAPFaultException ex) {
            // This is what we expected.
            return;
        }

        fail("Expected an exception here");
    }

    @Test
    public void testProxyRequestIdCardNotInCache() throws Exception {
        ServiceStubClient client = new ServiceStubClient("localhost:8586");

        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();
        securityHeader.getAssertion().getSubject().getNameID().setValue("not-in-cache");

        try {
            client.doInvoke(null, securityHeader, "", 0, 0, SERVER_URL + "/proxy/soap-request", false);
        } catch (SOAPFaultException e) {
            String faultCode = ((DetailEntry) e.getFault().getDetail().getDetailEntries().next()).getValue();
            assertEquals(Constants.FAULTCODE_NO_VALID_IDCARD_IN_CACHE, faultCode);
            assertEquals("Login required", e.getMessage());
        }
    }

    @Test
    public void testProxyRequestIdCardNotInCacheMalformedIdCard() throws Exception {
        ServiceStubClient client = new ServiceStubClient("localhost:8586");

        Security securityHeader = IdCardTestUtils.makeIDCardInSecurity();
        securityHeader.getAssertion().getSubject().getNameID().setValue("not-in-cache");
        securityHeader.getAssertion().getAttributeStatement().get(0).getAttribute().add(IdCardTestUtils.makeAttribute(MedcomAttributes.USER_EMAIL_ADDRESS, null, null));

        try {
            client.doInvoke(null, securityHeader, "", 0, 0, SERVER_URL + "/proxy/soap-request", false);
        } catch (SOAPFaultException e) {
            String faultCode = ((DetailEntry) e.getFault().getDetail().getDetailEntries().next()).getValue();
            assertEquals(Constants.FAULTCODE_MISSING_USER_IDCARD_IN_REQUEST, faultCode);
            assertTrue(e.getMessage().contains("No valid UserIDCard in request"));
        }
    }
}
